/***************************************************************************
** $Id: tork_mnu.cpp,v 1.3 2007/07/07 20:30:11 hoganrobert Exp $
 *   Copyright (C) 2006 by Robert Hogan   *
 *   robert@roberthogan.net   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <stdlib.h>
#include <dirent.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/stat.h>

#include <qdir.h>
#include <qfileinfo.h>
#include <qstringlist.h>

#include <kapplication.h>
#include <kglobal.h>
#include <kiconloader.h>
#include <kio/global.h>
#include <klocale.h>
#include <krun.h>
#include <kshell.h>
#include <ksimpleconfig.h>
#include <kstandarddirs.h>
#include <dcopref.h>
#include <dcopclient.h>
#include <kdebug.h>
#include <klocale.h>
#include <krun.h>

#include "tork_mnu.h"
#include "../libwhich/which.h"

K_EXPORT_KICKER_MENUEXT(tork, TorkMenu)

QValueVector<QString> torkifyApp(8);


TorkMenu::TorkMenu(QWidget *parent, const char *name, const QStringList& /* args */)
    : KPanelMenu("", parent, name),m_kdestate(true)
{
    p_dcopServer= new DCOPClient();
    p_dcopServer->attach ();

}

TorkMenu::~TorkMenu()
{
    KGlobal::locale()->removeCatalogue("libkickermenu_tork");
    p_dcopServer->detach();
    delete p_dcopServer;

}

void TorkMenu::initialize()
{
    if (initialized())
    {
        clear();
    }
    else
    {
        kapp->iconLoader()->addAppDir("tork");
    }

    setInitialized(true);

    torkifyApp[5] = "Konversation";
    torkifyApp[2] = "Kopete";
    torkifyApp[4] = "Gaim";
    torkifyApp[3] = "Pidgin";
    torkifyApp[6] = "Konsole";
    torkifyApp[1] = "Opera";
    torkifyApp[0] = "Firefox";
    torkifyApp[7] = "Email";

    char *firefox = NULL;
    char *opera = NULL;
    char *pidgin = NULL;
    char *konversation = NULL;
    char *gaim = NULL;
    char *konsole = NULL;
    char *kopete = NULL;

    QString paths = (QString) getenv("PATH");
    paths += ":/usr/local/sbin:/usr/sbin:/sbin:/usr/local/bin:/usr/bin:/bin";
    paths.replace("::",":");
    const char *path_list = paths;

    int path_index = 0;
    firefox = find_command_in_path("firefox", path_list, &path_index);

    path_index = 0;
    opera = find_command_in_path("opera", path_list, &path_index);

    path_index = 0;
    pidgin = find_command_in_path("pidgin", path_list, &path_index);

    path_index = 0;
    konversation = find_command_in_path("konversation", path_list, &path_index);

    path_index = 0;
    gaim = find_command_in_path("gaim", path_list, &path_index);

    path_index = 0;
    konsole = find_command_in_path("konsole", path_list, &path_index);

    path_index = 0;
    kopete = find_command_in_path("kopete", path_list, &path_index);

    if (firefox)
        insertItem( SmallIcon("tork_firefox"), i18n( "Anonymous Firefox" ), 0,1 );
    if (opera)
        insertItem( SmallIcon("tork_opera"), i18n( "Anonymous Opera" ), 1,2 );
    if (kopete)
        insertItem( SmallIcon("kopete"), i18n( "Anonymous Kopete" ), 2,3 );
    if (pidgin)
        insertItem( SmallIcon("pidgin"), i18n( "Anonymous Pidgin" ), 3,4 );
    if (gaim)
        insertItem( SmallIcon("gaim"), i18n( "Anonymous Gaim" ), 4,5 );
    if (konversation)
        insertItem( SmallIcon("konversation"), i18n( "Anonymous Konversation" ), 5,6 );
    if (konsole)
        insertItem( SmallIcon("tork_konsole"), i18n( "Anonymous Konsole Session" ), 6,7 );
    insertSeparator();

    
    insertItem( SmallIcon("tork_mail"), i18n( "Send Anonymous Email" ), 7,8 );

    insertSeparator();
    
    insertItem( SmallIcon("tork_konqueroron"), i18n( "Anonymize KDE" ), 9,9 );

    connect( this, SIGNAL( aboutToShow() ), SLOT( showPopup() ));

}

void TorkMenu::showPopup()
{
    bool anonymized = false;


    if (p_dcopServer->isApplicationRegistered ("tork"))
    {
        DCOPRef tork( "tork", "DCOPTork" );
        anonymized = tork.call( "getKDESetting" );
    }

    if (anonymized)
        changeItem(9, SmallIcon("tork_konqueroroff"), i18n( "De-Anonymize KDE" ));
    else
        changeItem(9, SmallIcon("tork_konqueroron"), i18n( "Anonymize KDE" ));


}

void TorkMenu::slotExec(int id)
{

    m_torkrunning = false;
    if (p_dcopServer->isApplicationRegistered ("tork"))
            m_torkrunning = true;
    

	switch(id)
	{
	case 9:
		anonymizeKDE(); break;
	default:
		anonymousApp(id);break;
	}

}

void TorkMenu::anonymousApp(int id)
{
    
    QString app = torkifyApp[id];
    app.prepend("anonymous");

    if (m_torkrunning){
        DCOPRef("tork*", "DCOPTork").send("startEverything");
        DCOPRef("tork*", "DCOPTork").send(QCString(app));
    }else
        KRun::runCommand( QString("tork --%1").arg(app), "tork", "tork" );

}


void TorkMenu::anonymizeKDE()
{
    if (m_torkrunning){
        DCOPRef("tork*", "DCOPTork").send("startEverything");
        DCOPRef("tork*", "DCOPTork").send("toggleKDESetting");
    }else
        KRun::runCommand( "tork --toggleKDE", "tork", "tork" );

//     if (m_kdestate)
//         changeItem(9, SmallIcon("tork_konqueroroff"), i18n( "De-Anonymize KDE" ));
//     else
//         changeItem(9, SmallIcon("tork_konqueroron"), i18n( "Anonymize KDE" ));
// 
//     m_kdestate = !m_kdestate;

}

#include "tork_mnu.moc"
