/***************************************************************************
** $Id: warnings.h,v 1.23 2007/08/14 18:20:30 hoganrobert Exp $
 *   Copyright (C) 2006 by Robert Hogan   *
 *   robert@roberthogan.net   *
 *   Based on config method in Tor
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "tork.h"


typedef struct message_t {
  const char *logmessage; /**< The full keyword (case insensitive). */
  const char *headline; /**< String (or null) describing initial value. */
  char torsaid[250]; /**< String (or null) describing initial value. */
  const char *body; /**< String (or null) describing initial value. */
  const char *type; /**< String (or null) describing initial value. */
  bool (tork::*pt2Member)();
  display_status_t state;  /**< The type of the value. */
  bool always;  /**< The type of the value. */
} message_t;


#define MSG(logmessage,headline,torsaid, body, type, show, state, always)                             \
  { logmessage, headline,torsaid,body, type, show, state, always }

static message_t _tork_messages[] = {
    MSG("ommencing hibernation", 
        "Tor Is No Longer Accepting Traffic!",
        "",
        "Tor has exceeded the bandwidth limits you set in "
        "'My Server->Performance' and so will no longer accept traffic. To fix this, set a higher threshold in"
        "'My Server->Performance'.",
        "Usage Warnings",
        (&tork::showUsage),
        DISPLAY_NONE, false),
    MSG("Accounting period ended", 
        "Tor Is No Longer Accepting Traffic!",
        "",
        "Tor has exceeded the bandwidth limits you set in "
        "'My Server->Performance' and so will no longer accept traffic. To fix this, set a higher threshold in"
        "'My Server->Performance'.",
        "Usage Warnings",
        (&tork::showUsage),
        DISPLAY_NONE, false),
    MSG("Hibernation period ended", 
        "Tor Is Now Accepting Traffic Again!",
        "",
        "Tor has completed a hibernation period that resulted from the settings you defined in "
        "'My Server->Performance'. If you do not want to accept traffic, set a lower threshold in"
        "'My Server->Performance'.",
        "Usage Warnings",
        (&tork::showUsage),
        DISPLAY_NONE, false),
    MSG("Your IP seems to have changed", 
        "You are running a server without any contact information!",
        "",
        "You can set your contact info in the configuration section 'My Server'. Please do so! ",
        "Usage Warnings",
        (&tork::showUsage),
        DISPLAY_NONE, false),
    MSG("jdsklajdkslajdskla",
        "Tor Controller is Not Responding!", 
        "",
        "TorK hasn't been able to contact Tor yet. If the display remains empty, stop and restart Tor via the control buttons. ",
        "Usage Warnings",
        (&tork::showUsage),
        DISPLAY_NONE, false),
    MSG("notorexecutable", 
        "Can't Find Your Tor Installation!",
        "",
        "You need to tell me where Tor is - it's not in any of your executable paths. Run the 'First Run Wizard' again from the 'Tools' menu.",
        "Usage Warnings",
        (&tork::showUsage),
        DISPLAY_NONE, false),
    MSG("noprivoxyexecutable", 
        "Can't Find Your Privoxy Installation!",
        "",
        "You need to tell me where Privoxy is - it's not in any of your executable paths. Run the 'First Run Wizard' again from the 'Tools' menu.",
        "General Warnings",
        (&tork::showUsage),
        DISPLAY_NONE, false),
    MSG("dnsrequestsdetected", 
        "You May Be Leaking DNS Requests!",
        "",
        "You should inspect the 'Traffic Log' to establish which application made the DNS request. Look for :domain or :53 in the Host/Port column of the Non-Tor traffic log. It may be that the application submitting the request is not of interest to you.",
        "DNS Leak Warnings",
        (&tork::showDNSLeaks),
        DISPLAY_NONE, false),
    MSG("Error writing to", 
        "Problem Accessing Files!",
        "",
        "You should make sure that you have the requisite access to the files required by Tor. Try typing 'tor' at the command line to investigate further.",
        "General Warnings",
        (&tork::showUsage),
        DISPLAY_NONE, false),
    MSG("cannot be read", 
        "Problem Accessing Files!",
        "",
        "You should make sure that you have the requesite access to the files required by Tor. Try typing 'tor' at the command line to investigate further.",
        "General Warnings",
        (&tork::showUsage),
        DISPLAY_NONE, false),
    MSG("Could not chmod", 
        "Problem Accessing Files!",
        "",
        "You should make sure that you have the requesite access to the files required by Tor. Try typing 'tor' at the command line to investigate further.",
        "General Warnings",
        (&tork::showUsage),
        DISPLAY_NONE, false),
    MSG("Couldn't open", 
        "Problem Accessing Files!",
        "",
        "You should make sure that you have the requesite access to the files required by Tor. Try typing 'tor' at the command line to investigate further.",
        "General Warnings",
        (&tork::showUsage),
        DISPLAY_NONE, false),
    MSG("Error creating", 
        "Problem Accessing Files!",
        "",
        "You should make sure that you have the requesite access to the files required by Tor. Try typing 'tor' at the command line to investigate further.",
        "General Warnings",
        (&tork::showUsage),
        DISPLAY_NONE, false),
    MSG("deprecated", 
        "TorK is using a deprecated config option!",
        "",
        "Please report this using 'Help->Report Bug' in the menu. Try to provide as much detail as possible. Thanks!",
        "Usage Warnings",
        (&tork::showUsage),
        DISPLAY_NONE, false),
    MSG("Failed to parse/validate config", 
        "TorK has passed an invalid configuration file to Tor!",
        "",
        "Please report this using 'Help->Report Bug' in the menu. Try to provide as much detail as possible. Thanks!",
        "General Warnings",
        (&tork::showUsage),
        DISPLAY_NONE, false),
    MSG("your clock just jumped", 
        "Tor is having problems with your local clock!",
        "",
        "Please report this using 'Help->Report Bug' in the menu or directly to or-talk@freehaven.net. Try to provide as much detail as possible. Thanks!",
        "Usage Warnings",
        (&tork::showUsage),
        DISPLAY_NONE, false),
    MSG("your ORPort is reachable from the outside", 
        "Your Tor Server appears to be working!!",
        "",
        "You are now serving the Tor network. Be careful out there!",
        "Usage Warnings",
        (&tork::showUsage),
        DISPLAY_NONE, false),
    MSG("your DirPort is reachable from the outside", 
        "Your Tor Server's directory appears to be working!!",
        "",
        "You can now serve directory listings to the Tor network. Be careful out there!",
        "Usage Warnings",
        (&tork::showUsage),
        DISPLAY_NONE, false),
    MSG("torclosedconnection", 
        "Tor Stopped Talking To Us!!",
        "",
        "Try starting Tork again!",
        "Usage Warnings",
        (&tork::showUsage),
        DISPLAY_NONE, false),
    MSG("hidden service is unavailable", 
        "The Hidden Service You Are Trying to Reach is Currently Unavailable!",
        "",
        "It's not just you. The hidden service you're trying to reach is actually down.",
        "Usage Warnings",
        (&tork::showUsage),
        DISPLAY_NONE, false),
    MSG("we have none", 
        "Tor can't retrieve a list of all servers on the network yet!",
        "",
        "TorK will try again as soon as Tor says it has enough info, in the meantime you can still use Tor though the servers list in the 'Tor Network' tab will be empty.",
        "Usage Warnings",
        (&tork::showUsage),
        DISPLAY_NONE, false),
    MSG("/var/lib/tor", 
        "You are using an old version of Tor that TorK is not compatible with!",
        "",
        "Try upgrading Tor through Tools->Download Tor.",
        "Usage Warnings",
        (&tork::showUsage),
        DISPLAY_NONE, false),
    MSG("cantreadprofiles", 
        "The file is not readable by Tork!",
        "",
        "Does it exist?.",
        "Usage Warnings",
        (&tork::showUsage),
        DISPLAY_NONE, false),
    MSG("cantcopyprofile", 
        "The file is not readable by Tork!",
        "",
        "Does it exist?",
        "Usage Warnings",
        (&tork::showUsage),
        DISPLAY_NONE, false),
    MSG("cantwritefirefoxsettings", 
        "The file is not writeable by Tork!",
        "",
        "Try again maybe.",
        "Usage Warnings",
        (&tork::showUsage),
        DISPLAY_NONE, false),
    MSG("torrunning", 
        "TorK is connected to Tor. You need to click 'Stop' first!",
        "",
        "To run the setup wizard, click 'Stop' in the Anonymize tab and try again.",
        "Usage Warnings",
        (&tork::showUsage),
        DISPLAY_NONE, false),

    MSG("bwreset", 
        "TorK has reset the bandwidth rates on Tor as per your instructions!",
        "",
        "You instructed TorK to do this in 'My Bandwidth'.",
        "Usage Warnings",
        (&tork::showUsage),
        DISPLAY_NONE, false),

    MSG("nonanonkonqhidden", 
        "TorK has hidden your non-anonymous Konqueror windows.",
        "",
        "Konqueror windows that have been used non-anonymously are not suitable for anonymous work!",
        "Usage Warnings",
        (&tork::showUsage),
        DISPLAY_NONE, true),

    MSG("nonanonkonqshown", 
        "TorK has un-hidden your non-anonymous Konqueror windows.",
        "",
        "Konqueror windows that were used non-anonymously are safe to use again for non-anonymous work!",
        "Usage Warnings",
        (&tork::showUsage),
        DISPLAY_NONE, true),

    MSG("DANGEROUS_VERSION", 
        "Your version of Tor may have problems.",
        "",
        "You should think of using the recommended version of Tor!",
        "Usage Warnings",
        (&tork::showUsage),
        DISPLAY_NONE, true),

    MSG("CIRCUIT_ESTABLISHED", 
        "Tor is ready for use as a client.",
        "",
        "You can now use Tor to anonymize your traffic!",
        "Usage Warnings",
        (&tork::showUsage),
        DISPLAY_NONE, true),

    MSG("TOO_MANY_CONNECTIONS", 
        "Your system has too many open connections.",
        "",
        "You should try running 'ulimit -n 10000' to improve things.",
        "Usage Warnings",
        (&tork::showUsage),
        DISPLAY_NONE, true),

    MSG("cookienotfound", 
        "Tor's Authentication Cookie Not Available.",
        "",
        "If you stored it elsewhere, please copy it to the suggested location above.",
        "Usage Warnings",
        (&tork::showUsage),
        DISPLAY_NONE, true),

    MSG("authenticationfailed", 
        "You used the wrong password to connect to Tor!",
        "",
        "Check the password entered in 'My Client'.",
        "Usage Warnings",
        (&tork::showUsage),
        DISPLAY_NONE, true),

    MSG("authenticationrequired", 
        "You need to use a password or cookie to connect to Tor!",
        "",
        "Enter the correct password or select cookie authentication in 'My Client'.",
        "Usage Warnings",
        (&tork::showUsage),
        DISPLAY_NONE, true),

  { NULL,  NULL, "", NULL, NULL, NULL, DISPLAY_NONE, false }
};
#undef MSG
